\
\ Code for the DCC booster.
\
\ $ProjectVersion: 0.52 $
\

hex

\ Port mapping

0 pin-b thermal              \ H bridge thermal sensor (in)
1 pin-b presence             \ Presence detected (out)
2 pin-b inversion            \ Inversion active (out)
3 pin-b disconnection        \ Disconnection (out)
4 pin-b dcc                  \ DCC signal (in)
5 pin-b jumper               \ Inversion allowed jumper (in)
6 pin-b loco-detected        \ Engine detected on track (in)
7 pin-b short-detected       \ Short-circuit detected (in)
1 pin-c brake                \ H bridge BRAKE (out)
2 pin-c pwm                  \ H bridge PWM (out)

variable inversion-mode      \ Is inversion enabled?

\ Output disabled is on portb3, H bridge control on portc1 and portc2

: enable-output ( -- )        \ PWM H, BRAKE L, DISCONN L
    brake bit-clr
    pwm bit-set
    disconnection bit-clr
;

: disable-output ( -- )       \ PWM L, BRAKE H, DISCONN H
    brake bit-set
    pwm bit-clr
    disconnection bit-set
 ;

\ Timer routines. At 10MhZ, with a prescaler of 2, each tick represent 0.8s.

: 100s ( -- )
    83 tmr0 !                     \ (0x100-0x83=0x7d, i.e. 0x7d*0.8s=100s)
    t0if intcon bit-clr           \ Clear overflow bit
    begin t0if intcon bit-set? until
;

: 300s ( -- ) 100s 100s 100s ;

\ Toggle inversion on portb2

: toggle-inversion ( -- ) inversion bit-toggle ;

: handle-shortcut ( -- )
    \ Toggle inversion if inversion is allowed
    inversion-mode @ if toggle-inversion then
    \ Wait for 100s
    100s
    \ If portb7 is now off, exit. Otherwise, disable the output and
    \ wait for 300s before reenabling it
    short-detected bit-set? /if exit then
    disable-output 300s enable-output
;

\ Maintain output disabled as long as thermal alert (portb0) is on

: handle-thermal-alert ( -- )
    disable-output
    begin 300s thermal bit-set? while repeat
    enable-output
;

\ If there is a shortcut on portb7, handle it

: mainloop ( -- )
    begin
	thermal bit-set? if handle-thermal-alert then
	short-detected bit-set? if handle-shortcut then
    again ;

\ Initialization

: init ( -- )
    0 portb !                                \ Clear portb output latches
    f1 trisb !                               \ Port b 1, 2 and 3 as outputs
    jumper bit-set? inversion-mode !         \ Inversion jumper
    0 portc !                                \ Clear portc output latches
    f9 trisc !                               \ Port c 1 and 2 as outputs
    0 option_reg !                           \ CLKOUT, prescaler(tmr0)=2
    enable-output
;

\ Main program
main
: main init mainloop ;

\ Configuration word

fosc-hs set-fosc    \ High-speed oscillator
